import { formatters } from 'conflux-web-core-helpers';
import { Accounts } from 'conflux-web-cfx-accounts';
import { ContractModuleFactory } from 'conflux-web-cfx-contract';
import { AbiCoder } from 'conflux-web-cfx-abi';
import { Network } from 'conflux-web-net';
import * as Utils from 'conflux-web-utils';
import ConfluxTx from 'confluxjs-transaction';
import { AbstractGetBlockMethod, AbstractMethod, AbstractGetBlockTransactionCountMethod, AbstractGetTransactionFromBlockMethod, SignTransactionMethod, SignMethod, GetAccountsMethod, GetNodeInfoMethod, GetProtocolVersionMethod, GetCoinbaseMethod, IsMiningMethod, GetHashrateMethod, IsSyncingMethod, GetGasPriceMethod, GetBalanceMethod, GetStorageAtMethod, GetCodeMethod, GetTransactionMethod, GetTransactionReceiptMethod, GetTransactionCountMethod, SendRawTransactionMethod, CfxSendTransactionMethod, CallMethod, EstimateGasMethod, SubmitWorkMethod, GetWorkMethod, GetPastLogsMethod, RequestAccountsMethod, VersionMethod, ChainIdMethod, GetEpochNumberMethod, AbstractMethodFactory } from 'conflux-web-core-method';
import isString from 'lodash/isString';
import isNumber from 'lodash/isNumber';
import deasync from 'deasync';
import { SyncingSubscription, NewPendingTransactionsSubscription, NewHeadsSubscription, LogSubscription } from 'conflux-web-core-subscriptions';
import { ProviderResolver } from 'conflux-web-providers';
import { AbstractConfluxWebModule } from 'conflux-web-core';

function asyncGeneratorStep(gen, resolve, reject, _next, _throw, key, arg) {
  try {
    var info = gen[key](arg);
    var value = info.value;
  } catch (error) {
    reject(error);
    return;
  }

  if (info.done) {
    resolve(value);
  } else {
    Promise.resolve(value).then(_next, _throw);
  }
}

function _asyncToGenerator(fn) {
  return function () {
    var self = this,
        args = arguments;
    return new Promise(function (resolve, reject) {
      var gen = fn.apply(self, args);

      function _next(value) {
        asyncGeneratorStep(gen, resolve, reject, _next, _throw, "next", value);
      }

      function _throw(err) {
        asyncGeneratorStep(gen, resolve, reject, _next, _throw, "throw", err);
      }

      _next(undefined);
    });
  };
}

function _classCallCheck(instance, Constructor) {
  if (!(instance instanceof Constructor)) {
    throw new TypeError("Cannot call a class as a function");
  }
}

function _defineProperties(target, props) {
  for (var i = 0; i < props.length; i++) {
    var descriptor = props[i];
    descriptor.enumerable = descriptor.enumerable || false;
    descriptor.configurable = true;
    if ("value" in descriptor) descriptor.writable = true;
    Object.defineProperty(target, descriptor.key, descriptor);
  }
}

function _createClass(Constructor, protoProps, staticProps) {
  if (protoProps) _defineProperties(Constructor.prototype, protoProps);
  if (staticProps) _defineProperties(Constructor, staticProps);
  return Constructor;
}

function _defineProperty(obj, key, value) {
  if (key in obj) {
    Object.defineProperty(obj, key, {
      value: value,
      enumerable: true,
      configurable: true,
      writable: true
    });
  } else {
    obj[key] = value;
  }

  return obj;
}

function _inherits(subClass, superClass) {
  if (typeof superClass !== "function" && superClass !== null) {
    throw new TypeError("Super expression must either be null or a function");
  }

  subClass.prototype = Object.create(superClass && superClass.prototype, {
    constructor: {
      value: subClass,
      writable: true,
      configurable: true
    }
  });
  if (superClass) _setPrototypeOf(subClass, superClass);
}

function _getPrototypeOf(o) {
  _getPrototypeOf = Object.setPrototypeOf ? Object.getPrototypeOf : function _getPrototypeOf(o) {
    return o.__proto__ || Object.getPrototypeOf(o);
  };
  return _getPrototypeOf(o);
}

function _setPrototypeOf(o, p) {
  _setPrototypeOf = Object.setPrototypeOf || function _setPrototypeOf(o, p) {
    o.__proto__ = p;
    return o;
  };

  return _setPrototypeOf(o, p);
}

function _isNativeReflectConstruct() {
  if (typeof Reflect === "undefined" || !Reflect.construct) return false;
  if (Reflect.construct.sham) return false;
  if (typeof Proxy === "function") return true;

  try {
    Date.prototype.toString.call(Reflect.construct(Date, [], function () {}));
    return true;
  } catch (e) {
    return false;
  }
}

function _assertThisInitialized(self) {
  if (self === void 0) {
    throw new ReferenceError("this hasn't been initialised - super() hasn't been called");
  }

  return self;
}

function _possibleConstructorReturn(self, call) {
  if (call && (typeof call === "object" || typeof call === "function")) {
    return call;
  }

  return _assertThisInitialized(self);
}

function _createSuper(Derived) {
  return function () {
    var Super = _getPrototypeOf(Derived),
        result;

    if (_isNativeReflectConstruct()) {
      var NewTarget = _getPrototypeOf(this).constructor;

      result = Reflect.construct(Super, arguments, NewTarget);
    } else {
      result = Super.apply(this, arguments);
    }

    return _possibleConstructorReturn(this, result);
  };
}

function _superPropBase(object, property) {
  while (!Object.prototype.hasOwnProperty.call(object, property)) {
    object = _getPrototypeOf(object);
    if (object === null) break;
  }

  return object;
}

function _get(target, property, receiver) {
  if (typeof Reflect !== "undefined" && Reflect.get) {
    _get = Reflect.get;
  } else {
    _get = function _get(target, property, receiver) {
      var base = _superPropBase(target, property);

      if (!base) return;
      var desc = Object.getOwnPropertyDescriptor(base, property);

      if (desc.get) {
        return desc.get.call(receiver);
      }

      return desc.value;
    };
  }

  return _get(target, property, receiver || target);
}

function set(target, property, value, receiver) {
  if (typeof Reflect !== "undefined" && Reflect.set) {
    set = Reflect.set;
  } else {
    set = function set(target, property, value, receiver) {
      var base = _superPropBase(target, property);

      var desc;

      if (base) {
        desc = Object.getOwnPropertyDescriptor(base, property);

        if (desc.set) {
          desc.set.call(receiver, value);
          return true;
        } else if (!desc.writable) {
          return false;
        }
      }

      desc = Object.getOwnPropertyDescriptor(receiver, property);

      if (desc) {
        if (!desc.writable) {
          return false;
        }

        desc.value = value;
        Object.defineProperty(receiver, property, desc);
      } else {
        _defineProperty(receiver, property, value);
      }

      return true;
    };
  }

  return set(target, property, value, receiver);
}

function _set(target, property, value, receiver, isStrict) {
  var s = set(target, property, value, receiver || target);

  if (!s && isStrict) {
    throw new Error('failed to set property');
  }

  return value;
}

var TransactionSigner = function () {
  function TransactionSigner(utils, formatters) {
    _classCallCheck(this, TransactionSigner);
    this.utils = utils;
    this.formatters = formatters;
  }
  _createClass(TransactionSigner, [{
    key: "sign",
    value: function () {
      var _sign = _asyncToGenerator( regeneratorRuntime.mark(function _callee(transaction, privateKey) {
        var cfxTx, validationResult, rlpEncoded, rawTransaction;
        return regeneratorRuntime.wrap(function _callee$(_context) {
          while (1) {
            switch (_context.prev = _context.next) {
              case 0:
                if (privateKey) {
                  _context.next = 2;
                  break;
                }
                throw new Error('No privateKey given to the TransactionSigner.');
              case 2:
                if (privateKey.startsWith('0x')) {
                  privateKey = privateKey.substring(2);
                }
                cfxTx = new ConfluxTx(transaction);
                cfxTx.sign(Buffer.from(privateKey, 'hex'));
                validationResult = cfxTx.validate(true);
                if (!(validationResult !== '')) {
                  _context.next = 8;
                  break;
                }
                throw new Error("TransactionSigner Error: ".concat(validationResult));
              case 8:
                rlpEncoded = cfxTx.serialize().toString('hex');
                rawTransaction = '0x' + rlpEncoded;
                return _context.abrupt("return", {
                  messageHash: Buffer.from(cfxTx.hash(false)).toString('hex'),
                  v: '0x' + Buffer.from(cfxTx.v).toString('hex'),
                  r: '0x' + Buffer.from(cfxTx.r).toString('hex'),
                  s: '0x' + Buffer.from(cfxTx.s).toString('hex'),
                  rawTransaction: rawTransaction
                });
              case 11:
              case "end":
                return _context.stop();
            }
          }
        }, _callee);
      }));
      function sign(_x, _x2) {
        return _sign.apply(this, arguments);
      }
      return sign;
    }()
  }, {
    key: "type",
    get: function get() {
      return 'TransactionSigner';
    }
  }]);
  return TransactionSigner;
}();

var GetBlockMethod = function (_AbstractGetBlockMeth) {
  _inherits(GetBlockMethod, _AbstractGetBlockMeth);
  var _super = _createSuper(GetBlockMethod);
  function GetBlockMethod(utils, formatters, moduleInstance) {
    _classCallCheck(this, GetBlockMethod);
    return _super.call(this, 'cfx_getBlockByEpochNumber', utils, formatters, moduleInstance);
  }
  _createClass(GetBlockMethod, [{
    key: "beforeExecution",
    value: function beforeExecution(moduleInstance) {
      if (this.isHash(this.parameters[0])) {
        this.rpcMethod = 'cfx_getBlockByHash';
      }
      _get(_getPrototypeOf(GetBlockMethod.prototype), "beforeExecution", this).call(this, moduleInstance);
    }
  }]);
  return GetBlockMethod;
}(AbstractGetBlockMethod);

var GetBlocksByEpochMethod = function (_AbstractMethod) {
  _inherits(GetBlocksByEpochMethod, _AbstractMethod);
  var _super = _createSuper(GetBlocksByEpochMethod);
  function GetBlocksByEpochMethod(utils, formatters, moduleInstance) {
    _classCallCheck(this, GetBlocksByEpochMethod);
    return _super.call(this, 'cfx_getBlocksByEpoch', 1, utils, formatters, moduleInstance);
  }
  _createClass(GetBlocksByEpochMethod, [{
    key: "beforeExecution",
    value: function beforeExecution(moduleInstance) {
      this.parameters[0] = this.formatters.inputBlockAddressFormatter(this.parameters[0]);
    }
  }]);
  return GetBlocksByEpochMethod;
}(AbstractMethod);

var GetBestBlockHashMethod = function (_AbstractMethod) {
  _inherits(GetBestBlockHashMethod, _AbstractMethod);
  var _super = _createSuper(GetBestBlockHashMethod);
  function GetBestBlockHashMethod(utils, formatters, moduleInstance) {
    _classCallCheck(this, GetBestBlockHashMethod);
    return _super.call(this, 'cfx_getBestBlockHash', 0, utils, formatters, moduleInstance);
  }
  return GetBestBlockHashMethod;
}(AbstractMethod);

var GetBlockByHashWithPivotAssumptionMethod = function (_AbstractMethod) {
  _inherits(GetBlockByHashWithPivotAssumptionMethod, _AbstractMethod);
  var _super = _createSuper(GetBlockByHashWithPivotAssumptionMethod);
  function GetBlockByHashWithPivotAssumptionMethod(utils, formatters, moduleInstance) {
    _classCallCheck(this, GetBlockByHashWithPivotAssumptionMethod);
    return _super.call(this, 'cfx_getBlockByHashWithPivotAssumption', 3, utils, formatters, moduleInstance);
  }
  return GetBlockByHashWithPivotAssumptionMethod;
}(AbstractMethod);

var GetBlockTransactionCountMethod = function (_AbstractGetBlockTran) {
  _inherits(GetBlockTransactionCountMethod, _AbstractGetBlockTran);
  var _super = _createSuper(GetBlockTransactionCountMethod);
  function GetBlockTransactionCountMethod(utils, formatters, moduleInstance) {
    _classCallCheck(this, GetBlockTransactionCountMethod);
    return _super.call(this, 'cfx_getBlockTransactionCountByNumber', utils, formatters, moduleInstance);
  }
  _createClass(GetBlockTransactionCountMethod, [{
    key: "beforeExecution",
    value: function beforeExecution(moduleInstance) {
      if (this.isHash(this.parameters[0])) {
        this.rpcMethod = 'cfx_getBlockTransactionCountByHash';
      }
      _get(_getPrototypeOf(GetBlockTransactionCountMethod.prototype), "beforeExecution", this).call(this, moduleInstance);
    }
  }]);
  return GetBlockTransactionCountMethod;
}(AbstractGetBlockTransactionCountMethod);

var GetTransactionFromBlockMethod = function (_AbstractGetTransacti) {
  _inherits(GetTransactionFromBlockMethod, _AbstractGetTransacti);
  var _super = _createSuper(GetTransactionFromBlockMethod);
  function GetTransactionFromBlockMethod(utils, formatters, moduleInstance) {
    _classCallCheck(this, GetTransactionFromBlockMethod);
    return _super.call(this, 'cfx_getTransactionByBlockAddressAndIndex', utils, formatters, moduleInstance);
  }
  _createClass(GetTransactionFromBlockMethod, [{
    key: "beforeExecution",
    value: function beforeExecution(moduleInstance) {
      if (this.isHash(this.parameters[0])) {
        this.rpcMethod = 'cfx_getTransactionByBlockHashAndIndex';
      }
      _get(_getPrototypeOf(GetTransactionFromBlockMethod.prototype), "beforeExecution", this).call(this, moduleInstance);
    }
  }]);
  return GetTransactionFromBlockMethod;
}(AbstractGetTransactionFromBlockMethod);

var CfxSignTransactionMethod = function (_SignTransactionMetho) {
  _inherits(CfxSignTransactionMethod, _SignTransactionMetho);
  var _super = _createSuper(CfxSignTransactionMethod);
  function CfxSignTransactionMethod(utils, formatters, moduleInstance) {
    _classCallCheck(this, CfxSignTransactionMethod);
    return _super.call(this, utils, formatters, moduleInstance);
  }
  _createClass(CfxSignTransactionMethod, [{
    key: "beforeExecution",
    value: function beforeExecution(moduleInstance) {
      this.parameters[0] = this.formatters.inputTransactionFormatter(this.parameters[0], moduleInstance);
    }
  }, {
    key: "execute",
    value: function execute() {
      if (this.parameters[0].storageLimit === undefined) {
        this.parameters[0]['storageLimit'] = 100000000;
      }
      if (this.parameters[0].chainId === undefined) {
        this.parameters[0]['chainId'] = 0;
      }
      if (this.parameters[0].epochHeight === undefined) {
        this.parameters[0].epochHeight = this.getEpochHeight();
      }
      if (isString(this.parameters[1])) {
        var account = this.moduleInstance.accounts.wallet[this.parameters[1]];
        if (account) {
          return this.moduleInstance.transactionSigner.sign(this.parameters[0], account.privateKey);
        }
      }
      if (isNumber(this.parameters[0].from)) {
        var _account = this.moduleInstance.accounts.wallet[this.parameters[0].from];
        if (_account) {
          return this.moduleInstance.transactionSigner.sign(this.parameters[0], _account.privateKey);
        }
      }
      return _get(_getPrototypeOf(CfxSignTransactionMethod.prototype), "execute", this).call(this);
    }
  }, {
    key: "getEpochHeight",
    value: function getEpochHeight() {
      var result;
      var isReturn = false;
      this.moduleInstance.currentProvider.send('cfx_epochNumber', []).then(function (epochNumber) {
        result = epochNumber;
        isReturn = true;
      });
      while (!isReturn) {
        deasync.runLoopOnce();
      }
      return result;
    }
  }]);
  return CfxSignTransactionMethod;
}(SignTransactionMethod);

var CfxSignMethod = function (_SignMethod) {
  _inherits(CfxSignMethod, _SignMethod);
  var _super = _createSuper(CfxSignMethod);
  function CfxSignMethod(utils, formatters, moduleInstance) {
    _classCallCheck(this, CfxSignMethod);
    return _super.call(this, utils, formatters, moduleInstance);
  }
  _createClass(CfxSignMethod, [{
    key: "execute",
    value: function execute() {
      if (this.moduleInstance.accounts.wallet[this.parameters[1]]) {
        return this.signLocally();
      }
      return _get(_getPrototypeOf(CfxSignMethod.prototype), "execute", this).call(this);
    }
  }, {
    key: "signLocally",
    value: function () {
      var _signLocally = _asyncToGenerator( regeneratorRuntime.mark(function _callee() {
        var signedMessage;
        return regeneratorRuntime.wrap(function _callee$(_context) {
          while (1) {
            switch (_context.prev = _context.next) {
              case 0:
                _context.prev = 0;
                this.beforeExecution(this.moduleInstance);
                signedMessage = this.moduleInstance.accounts.sign(this.parameters[1], this.moduleInstance.accounts.wallet[this.parameters[0]].privateKey);
                if (this.callback) {
                  this.callback(false, signedMessage);
                }
                return _context.abrupt("return", signedMessage);
              case 7:
                _context.prev = 7;
                _context.t0 = _context["catch"](0);
                if (this.callback) {
                  this.callback(_context.t0, null);
                }
                throw _context.t0;
              case 11:
              case "end":
                return _context.stop();
            }
          }
        }, _callee, this, [[0, 7]]);
      }));
      function signLocally() {
        return _signLocally.apply(this, arguments);
      }
      return signLocally;
    }()
  }]);
  return CfxSignMethod;
}(SignMethod);

var CfxGetAccountsMethod = function (_GetAccountsMethod) {
  _inherits(CfxGetAccountsMethod, _GetAccountsMethod);
  var _super = _createSuper(CfxGetAccountsMethod);
  function CfxGetAccountsMethod(utils, formatters, moduleInstance) {
    _classCallCheck(this, CfxGetAccountsMethod);
    return _super.call(this, utils, formatters, moduleInstance);
  }
  _createClass(CfxGetAccountsMethod, [{
    key: "execute",
    value: function execute() {
      if (this.moduleInstance.accounts.wallet.accountsIndex) {
        var accounts = [];
        for (var i = 0; i < this.moduleInstance.accounts.wallet.accountsIndex; i++) {
          accounts.push(this.moduleInstance.accounts.wallet[i].address);
        }
        return Promise.resolve(accounts);
      }
      return _get(_getPrototypeOf(CfxGetAccountsMethod.prototype), "execute", this).call(this);
    }
  }]);
  return CfxGetAccountsMethod;
}(GetAccountsMethod);

var MethodFactory = function (_AbstractMethodFactor) {
  _inherits(MethodFactory, _AbstractMethodFactor);
  var _super = _createSuper(MethodFactory);
  function MethodFactory(utils, formatters) {
    var _this;
    _classCallCheck(this, MethodFactory);
    _this = _super.call(this, utils, formatters);
    _this.methods = {
      getNodeInfo: GetNodeInfoMethod,
      getProtocolVersion: GetProtocolVersionMethod,
      getCoinbase: GetCoinbaseMethod,
      isMining: IsMiningMethod,
      getHashrate: GetHashrateMethod,
      isSyncing: IsSyncingMethod,
      getGasPrice: GetGasPriceMethod,
      getAccounts: CfxGetAccountsMethod,
      getBalance: GetBalanceMethod,
      getStorageAt: GetStorageAtMethod,
      getCode: GetCodeMethod,
      getBlock: GetBlockMethod,
      getBlockTransactionCount: GetBlockTransactionCountMethod,
      getTransaction: GetTransactionMethod,
      getTransactionFromBlock: GetTransactionFromBlockMethod,
      getTransactionReceipt: GetTransactionReceiptMethod,
      getTransactionCount: GetTransactionCountMethod,
      sendSignedTransaction: SendRawTransactionMethod,
      signTransaction: CfxSignTransactionMethod,
      sendTransaction: CfxSendTransactionMethod,
      sign: CfxSignMethod,
      call: CallMethod,
      estimateGas: EstimateGasMethod,
      submitWork: SubmitWorkMethod,
      getWork: GetWorkMethod,
      getPastLogs: GetPastLogsMethod,
      requestAccounts: RequestAccountsMethod,
      getId: VersionMethod,
      getChainId: ChainIdMethod,
      getEpochNumber: GetEpochNumberMethod,
      getBlocksByEpoch: GetBlocksByEpochMethod,
      getBlockByHashWithPivotAssumption: GetBlockByHashWithPivotAssumptionMethod,
      getBestBlockHash: GetBestBlockHashMethod
    };
    return _this;
  }
  return MethodFactory;
}(AbstractMethodFactory);

var SubscriptionsFactory = function () {
  function SubscriptionsFactory(utils, formatters) {
    _classCallCheck(this, SubscriptionsFactory);
    this.utils = utils;
    this.formatters = formatters;
  }
  _createClass(SubscriptionsFactory, [{
    key: "getSubscription",
    value: function getSubscription(moduleInstance, type, options) {
      switch (type) {
        case 'logs':
          return new LogSubscription(options, this.utils, this.formatters, moduleInstance, new GetPastLogsMethod(this.utils, this.formatters, moduleInstance));
        case 'newBlockHeaders':
          return new NewHeadsSubscription(this.utils, this.formatters, moduleInstance);
        case 'pendingTransactions':
          return new NewPendingTransactionsSubscription(this.utils, this.formatters, moduleInstance);
        case 'syncing':
          return new SyncingSubscription(this.utils, this.formatters, moduleInstance);
        default:
          throw new Error("Unknown subscription: ".concat(type));
      }
    }
  }]);
  return SubscriptionsFactory;
}();

var Cfx = function (_AbstractConfluxWebMo) {
  _inherits(Cfx, _AbstractConfluxWebMo);
  var _super = _createSuper(Cfx);
  function Cfx(provider, methodFactory, net, accounts, abiCoder, utils, formatters, subscriptionsFactory, contractModuleFactory, options, nodeNet) {
    var _this;
    _classCallCheck(this, Cfx);
    _this = _super.call(this, provider, options, methodFactory, nodeNet);
    _this.net = net;
    _this.accounts = accounts;
    _this.abi = abiCoder;
    _this.utils = utils;
    _this.formatters = formatters;
    _this.subscriptionsFactory = subscriptionsFactory;
    _this.contractModuleFactory = contractModuleFactory;
    _this.initiatedContracts = [];
    _this._transactionSigner = options.transactionSigner;
    _this.Contract = function (abi, address) {
      var options = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};
      var contract = _this.contractModuleFactory.createContract(_this.currentProvider, _this.accounts, abi, address, {
        defaultAccount: options.from || options.defaultAccount || _this.defaultAccount,
        defaultEpoch: options.defaultEpoch || _this.defaultEpoch,
        defaultGas: options.gas || options.defaultGas || _this.defaultGas,
        defaultGasPrice: options.gasPrice || options.defaultGasPrice || _this.defaultGasPrice,
        transactionBlockTimeout: options.transactionBlockTimeout || _this.transactionBlockTimeout,
        transactionConfirmationBlocks: options.transactionConfirmationBlocks || _this.transactionConfirmationBlocks,
        transactionPollingTimeout: options.transactionPollingTimeout || _this.transactionPollingTimeout,
        transactionSigner: _this.transactionSigner,
        data: options.data
      });
      _this.initiatedContracts.push(contract);
      return contract;
    };
    return _this;
  }
  _createClass(Cfx, [{
    key: "clearSubscriptions",
    value: function clearSubscriptions() {
      return _get(_getPrototypeOf(Cfx.prototype), "clearSubscriptions", this).call(this, 'cfx_unsubscribe');
    }
  }, {
    key: "subscribe",
    value: function subscribe(type, options, callback) {
      return this.subscriptionsFactory.getSubscription(this, type, options).subscribe(callback);
    }
  }, {
    key: "setProvider",
    value: function setProvider(provider, net) {
      var setContractProviders = this.initiatedContracts.every(function (contract) {
        return contract.setProvider(provider, net);
      });
      return this.net.setProvider(provider, net) && _get(_getPrototypeOf(Cfx.prototype), "setProvider", this).call(this, provider, net) && setContractProviders;
    }
  }, {
    key: "transactionSigner",
    get: function get() {
      return this._transactionSigner;
    }
    ,
    set: function set(transactionSigner) {
      if (transactionSigner.type && transactionSigner.type === 'TransactionSigner') {
        throw new Error('Invalid TransactionSigner given!');
      }
      this._transactionSigner = transactionSigner;
      this.accounts.transactionSigner = transactionSigner;
      this.initiatedContracts.forEach(function (contract) {
        contract.transactionSigner = transactionSigner;
      });
    }
  }, {
    key: "defaultGasPrice",
    set: function set(value) {
      this.initiatedContracts.forEach(function (contract) {
        contract.defaultGasPrice = value;
      });
      this.net.defaultGasPrice = value;
      _set(_getPrototypeOf(Cfx.prototype), "defaultGasPrice", value, this, true);
    }
    ,
    get: function get() {
      return _get(_getPrototypeOf(Cfx.prototype), "defaultGasPrice", this);
    }
  }, {
    key: "defaultGas",
    set: function set(value) {
      this.initiatedContracts.forEach(function (contract) {
        contract.defaultGas = value;
      });
      this.net.defaultGas = value;
      _set(_getPrototypeOf(Cfx.prototype), "defaultGas", value, this, true);
    }
    ,
    get: function get() {
      return _get(_getPrototypeOf(Cfx.prototype), "defaultGas", this);
    }
  }, {
    key: "transactionBlockTimeout",
    set: function set(value) {
      this.initiatedContracts.forEach(function (contract) {
        contract.transactionBlockTimeout = value;
      });
      this.net.transactionBlockTimeout = value;
      _set(_getPrototypeOf(Cfx.prototype), "transactionBlockTimeout", value, this, true);
    }
    ,
    get: function get() {
      return _get(_getPrototypeOf(Cfx.prototype), "transactionBlockTimeout", this);
    }
  }, {
    key: "transactionConfirmationBlocks",
    set: function set(value) {
      this.initiatedContracts.forEach(function (contract) {
        contract.transactionConfirmationBlocks = value;
      });
      this.net.transactionConfirmationBlocks = value;
      _set(_getPrototypeOf(Cfx.prototype), "transactionConfirmationBlocks", value, this, true);
    }
    ,
    get: function get() {
      return _get(_getPrototypeOf(Cfx.prototype), "transactionConfirmationBlocks", this);
    }
  }, {
    key: "transactionPollingTimeout",
    set: function set(value) {
      this.initiatedContracts.forEach(function (contract) {
        contract.transactionPollingTimeout = value;
      });
      this.net.transactionPollingTimeout = value;
      _set(_getPrototypeOf(Cfx.prototype), "transactionPollingTimeout", value, this, true);
    }
    ,
    get: function get() {
      return _get(_getPrototypeOf(Cfx.prototype), "transactionPollingTimeout", this);
    }
  }, {
    key: "defaultAccount",
    set: function set(value) {
      var _this2 = this;
      this.initiatedContracts.forEach(function (contract) {
        contract.defaultAccount = _this2.utils.toChecksumAddress(value);
      });
      this.net.defaultAccount = value;
      _set(_getPrototypeOf(Cfx.prototype), "defaultAccount", value, this, true);
    }
    ,
    get: function get() {
      return _get(_getPrototypeOf(Cfx.prototype), "defaultAccount", this);
    }
  }, {
    key: "defaultEpoch",
    set: function set(value) {
      this.initiatedContracts.forEach(function (contract) {
        contract.defaultEpoch = value;
      });
      this.net.defaultEpoch = value;
      _set(_getPrototypeOf(Cfx.prototype), "defaultEpoch", value, this, true);
    }
    ,
    get: function get() {
      return _get(_getPrototypeOf(Cfx.prototype), "defaultEpoch", this);
    }
  }]);
  return Cfx;
}(AbstractConfluxWebModule);

function TransactionSigner$1() {
  return new TransactionSigner(Utils, formatters);
}
function Cfx$1(provider) {
  var net = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : null;
  var options = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};
  if (!options.transactionSigner || options.transactionSigner.type === 'TransactionSigner') {
    options.transactionSigner = new TransactionSigner$1();
  }
  var resolvedProvider = new ProviderResolver().resolve(provider, net);
  var accounts = new Accounts(resolvedProvider, null, options);
  var abiCoder = new AbiCoder();
  return new Cfx(resolvedProvider, new MethodFactory(Utils, formatters), new Network(resolvedProvider, null, options), accounts, abiCoder, Utils, formatters, new SubscriptionsFactory(Utils, formatters), new ContractModuleFactory(Utils, formatters, abiCoder, accounts), options, net);
}

export { Cfx$1 as Cfx, TransactionSigner$1 as TransactionSigner };
